#!/bin/bash
#
#
# 	Script d'initialisation d'un dépot vMap versionné sous git
#
#   Split repo
#   git subtree split -P module/ -b module_split_branch
#
#   Add subtree
#   git subtree add --prefix vmap "git@gitlab.veremes.net:Development/vitis_apps/sources/Vitis.git" master
#
#

echo -n "Existing app path (path/empty)? "
read APP_EXISTING_PATH

# Récupère la liste des dépendances
. ./get_deps.sh

cd ..

# Supprime les anciens symlinks
if [ -d "client" ]; then
	rm -Rf client
fi
if [ -d "vas" ]; then
	rm -Rf vas
fi
find src/vitis/client -type l -delete
find src/vitis/vas/sql -type l -delete
find src/vitis/vas/rest -type l -delete

# Vitis
echo "---- Link vitis ----"
if ! [ -e "$(pwd)/client" ]; then

	mkdir client
	mkdir client/javascript
	mkdir client/modules
	ln -rsf src/vitis/client/css client/css
	ln -rsf src/vitis/client/forms client/forms
	ln -rsf src/vitis/client/images client/images
	ln -rsf src/vitis/client/javascript/app client/javascript/app
	ln -rsf src/vitis/client/javascript/externs client/javascript/externs
	ln -rsf src/vitis/client/javascript/require client/javascript/require
	ln -rsf src/vitis/client/lang client/lang
	ln -rsf src/vitis/client/less client/less
	ln -rsf src/vitis/client/modules/vitis client/modules/vitis
	ln -rsf src/vitis/client/templates client/templates
	ln -rsf src/vitis/client/.htaccess client/.htaccess
	ln -rsf src/vitis/client/index.html client/index.html
fi
if ! [ -e "$(pwd)/vas" ]; then

	mkdir vas
	mkdir vas/rest
	mkdir vas/rest/ws
	mkdir vas/rest/class
	ln -rsf src/vitis/vas/doc vas/doc
	ln -rsf src/vitis/vas/rest/inc vas/rest/inc
	ln -rsf src/vitis/vas/rest/ws/vitis vas/rest/ws/vitis
	ln -rsf src/vitis/vas/rest/index.phtml vas/rest/index.phtml
	ln -rsf src/vitis/vas/rest/index.vhtml vas/rest/index.vhtml
	ln -rsf src/vitis/vas/sql vas/sql
	ln -rsf src/vitis/vas/util vas/util

	if [ -e "src/vitis/vas/rest/class" ]; then
		for elem in $( ls -a "src/vitis/vas/rest/class"); do
		    if [[ -d "src/vitis/vas/rest/class/${elem}" || -f "src/vitis/vas/rest/class/${elem}" ]]; then
		        echo "---- Link src/vitis/vas/rest/class/$elem"
		        ln -rsf src/vitis/vas/rest/class/$elem vas/rest/class/$elem
		    fi
		done
	fi
fi

echo "---- Link conf ----"
if ! [ -e "src/vitis/client/conf" ]; then
	mkdir client/conf
	for elem in $( ls "conf"); do
		if [[ $elem != 'properties.json' ]]; then
			if [[ -d "conf/${elem}" || -f "conf/${elem}" ]]; then
				echo "---- Link conf/$elem"
				ln -rsf conf/$elem client/conf/$elem
			fi
		fi
	done
fi
if ! [ -e "conf/closure" ]; then
	ln -rsf src/closure/conf conf/closure
fi

# Modules
if [[ $depsCount > 0 ]]; then
	for (( i=1; i <= $depsCount; i++ )); do
		if [ ${aDeps[$i, nature]} == "modules" ]; then
			if [ ${aDeps[$i, name]:0:7} == "module_" ]; then
				module=${aDeps[$i, name]:7}
				echo "---- Link module_${module} ----"
				if [ -d "$(pwd)/src/module_${module}/module" ]; then
					if ! [ -e "src/vitis/client/modules/${module}" ]; then
						ln -rsf src/module_${module}/module client/modules/${module}
					fi
				fi
				if [ -d "$(pwd)/src/module_${module}/web_service/ws" ]; then
					if ! [ -e "src/vitis/vas/rest/ws/${module}" ]; then
						ln -rsf src/module_${module}/web_service/ws vas/rest/ws/${module}
					fi
				fi
				if [ -d "$(pwd)/src/module_${module}/web_service/sql" ]; then
					if ! [ -e "src/vitis/vas/sql/${module}" ]; then
						ln -rsf src/module_${module}/web_service/sql vas/sql/${module}
					fi
				fi
				if [ -d "$(pwd)/src/module_${module}/web_service/class" ]; then
					for class_dir in $( ls "$(pwd)/src/module_${module}/web_service/class"); do
						if [ -d "$(pwd)/src/module_${module}/web_service/class/${class_dir}" ]; then
							echo "---- Link module_${module}/${class_dir} ----"
							if ! [ -e "src/vitis/vas/rest/class/${class_dir}" ]; then
								ln -rsf src/module_${module}/web_service/class/${class_dir} vas/rest/class/${class_dir}
							fi
						fi
					done
				fi
			fi
		fi
	done

	# Utilisation d'une install déjà existante
	if [ -d "${APP_EXISTING_PATH}" ]; then
		echo "---- Link exixting app ${APP_EXISTING_PATH} ----"

		declare -a existing_folders_path=("vas/server"
		                "vas/rest/conf"
		                "vas/log"
						"vas/public"
						"vas/shared"
						"vas/tmp"
						"vas/upload"
		                "vas/ws_data")
		for folder_path in "${existing_folders_path[@]}"; do
			if [ -d "${APP_EXISTING_PATH}/$folder_path" ]; then
				echo "copy $folder_path"
				cp -Rf "${APP_EXISTING_PATH}/$folder_path" "$(pwd)/$folder_path"
				chmod 777 -R "$(pwd)/$folder_path"
			fi
		done

		declare -a existing_files_path=("client/conf/properties.json"
										"vas/rest/.htaccess")
		for file_path in "${existing_files_path[@]}"; do
			if [ -f "${APP_EXISTING_PATH}/$file_path" ]; then
				echo "copy $file_path"
				cp -f "${APP_EXISTING_PATH}/$file_path" "$(pwd)/$file_path"
			fi
		done

		for properties_dir in $( ls "${APP_EXISTING_PATH}/vas/rest/conf"); do
			if [ -d "${APP_EXISTING_PATH}/vas/rest/conf/${properties_dir}" ]; then
				echo "copy ${APP_EXISTING_PATH}/vas/rest/conf/${properties_dir}/ > $(pwd)/vas/rest/conf/"
				cp -Rf "${APP_EXISTING_PATH}/vas/rest/conf/${properties_dir}/" "$(pwd)/vas/rest/conf/"
			fi
		done
	fi
fi
